<?php 

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\Util\Functions;
use Zeedhi\Framework\DataSource\FilterCriteria;

class SelectCliente extends CrudWithParameterBag {
    
    protected $dataSourceName = 'select_cliente'; 
    protected $util;
    protected $dataSourceClienteMult = 'select_cliente_filialMultiplo';
    protected $dataSourceConsMult = 'consumidorClienteMult';
    protected $dataSourceSelectClientes = 'select_clientes_filiais';
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag,  $util) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this -> util = $util;
        
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'select_cliente';
        $this->parameterBagColumns = array('NRORG');
    	parent::find($request, $response);
    }
    
     public function findConsumidor(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'consumidorClienteMult';
        $this->parameterBagColumns = array('NRORG', 'CDCLIENTE', 'TCDCLIENTE'); 
        $conditions = $this->util->getParamsAsArray($request);
        
        $dataSourceFilter = array();
        $likeableConditions = $this->util->getLikeConditions($request);
        
        foreach ($likeableConditions as $condition) {
			if(Functions::arrayKeyExists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
		}
		
		unset($conditions['CDCONSUMIDOR']);
		
		if(!empty($conditions['CDCLIENTE'])){
		    $conditions = $this->util->buildParams('CDCLIENTE',$conditions);    
		} else {
		   $conditions['CDCLIENTE'] = null; 
		   $conditions['TCDCLIENTE'] = 'T'; 
		}
		
        $newRequest = $this->util->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        
        foreach ($likeableConditions as $condition) {
			$newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
		
        parent::find($newRequest, $response);
    }
    
    public function findConsumidorClienteTpCons(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'consumidorClienteMultTpCons';
        $this->parameterBagColumns = array('NRORG', 'CDCLIENTE', 'TCDCLIENTE', 'CDTIPOCONS', 'TCDTIPOCONS'); 
        $conditions = $this->util->getParamsAsArray($request);
        
        $dataSourceFilter = array();
        $likeableConditions = $this->util->getLikeConditions($request);
        
        foreach ($likeableConditions as $condition) {
			if(Functions::arrayKeyExists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
		}
		
		unset($conditions['CDCONSUMIDOR']);
		
		if(!empty($conditions['CDCLIENTE'])){
		    $conditions = $this->util->buildParams('CDCLIENTE',$conditions);    
		} else {
		   $conditions['CDCLIENTE'] = null; 
		   $conditions['TCDCLIENTE'] = 'T'; 
		}
		
		if(!empty($conditions['CDTIPOCONS'])){
		    $conditions = $this->util->buildParams('CDTIPOCONS',$conditions);
		} else {
		   $conditions['CDTIPOCONS'] = null; 
		   $conditions['TCDTIPOCONS'] = 'T'; 
		}
		
        $newRequest = $this->util->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        
        foreach ($likeableConditions as $condition) {
			$newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
		
        parent::find($newRequest, $response);
    }

    public function findClienteMult(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'select_cliente_filialMultiplo';
        $this->parameterBagColumns = array('NRORG', 'CDFILIAL', 'TCDFILIAL');
        $conditions = $this->util->getParamsAsArray($request);
        
        $dataSourceFilter = array();
        $likeableConditions = $this->util->getLikeConditions($request);
        
        foreach ($likeableConditions as $condition) {
			if(Functions::arrayKeyExists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
		}
		
        $conditions = $this->util->buildParams('CDFILIAL',$conditions);
        $newRequest = $this->util->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        
        foreach ($likeableConditions as $condition) {
			$newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
		
        parent::find($newRequest, $response);
    }
    
    public function findClientes(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        
        $this->dataSourceName = 'select_clientes_filiais';
        $this->parameterBagColumns = array('NRORG', 'CDFILIAL');
        
        $conditions = $this->util->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);
        
        if(isset($conditions['CDFILIAL']) && !empty($conditions['CDFILIAL'])) {
            foreach($conditions as $key => $value) {
                $dataSourceFilter[] = $condition = array(
                    "columnName" => $key,
                    "operator" => "=",
                    "value" => $value
                );
            }
            
            // para o CTRL+F funcionar
            for($i = 0; $i < count($conditions); $i++) {
                if(strpos($dataSourceFilter[$i]["columnName"], '|') != false) {
                    $dataSourceFilter[$i]["operator"] = "MAPPED_LIKE_ALL";
                }
            }
            
            $newRequest = new DTO\Request\Filter (
                new FilterCriteria(
                    $request->getFilterCriteria()->getDataSourceName(),
                    $dataSourceFilter,
                    $request->getFilterCriteria()->getPage(),
                    $request->getFilterCriteria()->getPageSize()
                ),
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            );
            
            $result = parent::find($newRequest, $response);
        }
        
        return $result;
    }
    
    private function buildParams($conditions) {
        if(!isset($conditions['CDFILIAL']) || $conditions['CDFILIAL'] == "") {
            $conditions['CDFILIAL'] = '';
        } else {
            $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
        }
        
        if(isset($conditions['NRORG'])) {
            $conditions['NRORG'] = $conditions['NRORG'];
        }
        
        return $conditions;
    }
}